@@ -4,7 +4,6 @@ import inspect
 import logging
 import shutil
 import time
-import warnings
 from abc import ABC, abstractmethod
 from contextlib import contextmanager
 from copy import deepcopy
@@ -216,6 +215,7 @@ def _sample_config(optimization_dir, sampler, serializer, logger):
     base_result_directory = optimization_dir / "results"
     if pending_configs_free:
         logger.debug("Sampling a pending config without a worker")
+        is_continuation_of_crashed_config = True
         config_id, config = more_itertools.first(pending_configs_free.items())
         config_working_directory = base_result_directory / f"config_{config_id}"
         previous_config_id_file = config_working_directory / "previous_config.id"
@@ -225,6 +225,7 @@ def _sample_config(optimization_dir, sampler, serializer, logger):
             previous_config_id = None
     else:
         logger.debug("Sampling a new configuration")
+        is_continuation_of_crashed_config = False
         sampler.load_results(previous_results, pending_configs)
         config, config_id, previous_config_id = sampler.get_config_and_ids()
 
@@ -249,7 +250,13 @@ def _sample_config(optimization_dir, sampler, serializer, logger):
     serializer.dump(config, config_working_directory / "config")
 
     logger.debug(f"Sampled config {config_id}")
-    return config_id, config, config_working_directory, previous_working_directory
+    return (
+        config_id,
+        config,
+        config_working_directory,
+        previous_working_directory,
+        is_continuation_of_crashed_config,
+    )
 
 
 def _evaluate_config(
@@ -260,12 +267,11 @@ def _evaluate_config(
     previous_working_directory,
     logger,
 ):
-    # First, the actual evaluation along with error handling and support of multiple APIs
     config = deepcopy(config)
     logger.info(f"Start evaluating config {config_id}")
     try:
-        # API support: If working_directory and previous_working_directory are included
-        # in the signature we supply their values, otherwise we simply do nothing.
+        # If working_directory and previous_working_directory are included in the
+        # signature we supply their values, otherwise we simply do nothing.
         evaluation_fn_params = inspect.signature(evaluation_fn).parameters
         directory_params = []
         if "working_directory" in evaluation_fn_params:
@@ -273,28 +279,27 @@ def _evaluate_config(
         if "previous_working_directory" in evaluation_fn_params:
             directory_params.append(previous_working_directory)
 
-        # API support: Allow config to be used as:
-        try:
-            # 1. Individual keyword arguments
-            # 2. Allowed to be captured as **configs
-            result = evaluation_fn(
-                *directory_params,
-                **config,
-            )
-        except TypeError:  # TODO : remove this part (deprecated part)
-            # 3. As a mere single keyword argument
-            result = evaluation_fn(
-                *directory_params,
-                config=config,
-            )
-            warnings.warn(
-                "Using the config argument for the evaluation function will"
-                "soon be removed. Please use keyword arguments, or catch the"
-                "config with '**config'.",
-                FutureWarning,
-            )
-        if not isinstance(result, dict):
-            raise ValueError("The evaluation result should be a dictionnary")
+        result = evaluation_fn(
+            *directory_params,
+            **config,
+        )
+
+        # Ensuring the result have the correct format that can be exploited by other functions
+        if isinstance(result, dict):
+            try:
+                result["loss"] = float(result["loss"])
+            except KeyError as e:
+                raise ValueError("The loss should value should be provided") from e
+            except (TypeError, ValueError) as e:
+                raise ValueError("The loss should be a float") from e
+        else:
+            try:
+                result = float(result)
+            except (TypeError, ValueError) as e:
+                raise ValueError(
+                    "The evaluation result should be a dictionnary or a float"
+                ) from e
+            result = {"loss": result}
     except Exception:
         logger.exception(
             f"An error occured during evaluation of config {config_id}: " f"{config}."
@@ -317,6 +322,7 @@ def run(
     logger=None,
     post_evaluation_hook=None,
     overwrite_optimization_dir=False,
+    filesystem_grace_period_for_crashed_configs=45,
 ):
     serializer_cls = instance_from_map(
         SerializerMapping, serializer, "serializer", as_class=True
@@ -375,6 +381,7 @@ def run(
                         config,
                         working_directory,
                         previous_working_directory,
+                        is_continuation_of_crashed_config,
                     ) = _sample_config(optimization_dir, sampler, serializer, logger)
 
                 config_lock_file = working_directory / ".config_lock"
@@ -384,6 +391,20 @@ def run(
             finally:
                 decision_locker.release_lock()
 
+            if is_continuation_of_crashed_config:
+                # This is to catch the case of the shared filesystem being out of sync,
+                # so the lock is not there anymore, but the result is not yet seen by one
+                # of the workers. Wait for a grace period and then check if the config
+                # really crashed.
+                logger.info(
+                    f"Checking if config {config_id} crashed and needs to be continued."
+                )
+                time.sleep(filesystem_grace_period_for_crashed_configs)
+                if non_empty_file(working_directory / f"result{serializer_cls.SUFFIX}"):
+                    logger.info(f"Config {config_id} did not crash.")
+                    continue
+                logger.info(f"Config {config_id} did crash, so continuing/restarting it")
+
             if config_lock_acquired:
                 try:
                     # 1. First, we evaluate the config
