@@ -9,9 +9,15 @@ from pathlib import Path
 from typing import Any, Callable
 
 import dill
+import numpy as np
+import torch
 import yaml
 
 
+def non_empty_file(file_path: Path) -> bool:
+    return file_path.exists() and file_path.stat().st_size != 0
+
+
 def find_files(
     directory: Path, files: list[str], any_suffix=False, check_nonempty=False
 ) -> list[Path]:
@@ -23,7 +29,7 @@ def find_files(
         for f_path in glob.glob(pattern, recursive=True):
             path_found = Path(f_path)
             if path_found.is_file():
-                if check_nonempty and path_found.stat().st_size == 0:
+                if check_nonempty and not non_empty_file(path_found):
                     continue
                 found_paths.append(path_found)
     return found_paths
@@ -32,66 +38,99 @@ def find_files(
 # Serializers
 
 
+def get_data_representation(data: Any):
+    """Common data representations. Other specific types should be handled
+    by the user in his Parameter class."""
+    if isinstance(data, dict):
+        return {key: get_data_representation(val) for key, val in data.items()}
+    elif isinstance(data, list) or isinstance(data, tuple):
+        return [get_data_representation(val) for val in data]
+    elif type(data).__module__ == np.__name__ or isinstance(data, torch.Tensor):
+        data = data.tolist()
+        if type(data).__module__ == np.__name__:
+            data = data.item()
+        return get_data_representation(data)
+    elif hasattr(data, "serialize"):
+        return get_data_representation(data.serialize())
+    else:
+        return data
+
+
 class DataSerializer:
     SUFFIX = ""
+    PRE_SERIALIZE = True
 
     def __init__(self, config_loader: Callable | None = None):
         self.config_loader = config_loader or (lambda x: x)
 
     @abstractmethod
-    def load(self, path: Path | str):
+    def _load_from(self, path: str):
         raise NotImplementedError
 
-    def load_config(self, path: Path | str):
-        return self.config_loader(self.load(path))
-
     @abstractmethod
-    def dump(self, data: Any, path: Path | str):
+    def _dump_to(self, data: Any, path: str):
         raise NotImplementedError
 
+    def load(self, path: Path | str, add_suffix=True):
+        path = str(path)
+        if add_suffix and Path(path).suffix != self.SUFFIX:
+            path = path + self.SUFFIX
+        return self._load_from(path)
+
+    def dump(self, data: Any, path: Path | str, add_suffix=True):
+        if self.PRE_SERIALIZE:
+            data = get_data_representation(data)
+        path = str(path)
+        if add_suffix and Path(path).suffix != self.SUFFIX:
+            path = path + self.SUFFIX
+        self._dump_to(data, path)
+
+    def load_config(self, path: Path | str):
+        if self.PRE_SERIALIZE:
+            return self.config_loader(self.load(path))
+        return self.load(path)
+
 
 class DillSerializer(DataSerializer):
     SUFFIX = ".dill"
+    PRE_SERIALIZE = False
 
-    def load(self, path: Path | str):
-        with open(str(path), "rb") as file_stream:
+    def _load_from(self, path: str):
+        with open(path, "rb") as file_stream:
             return dill.load(file_stream)
 
-    def load_config(self, path: Path | str):
-        # The object is already built
-        return self.load(path)
-
-    def dump(self, data: Any, path: Path | str):
-        with open(str(path), "wb") as file_stream:
+    def _dump_to(self, data: Any, path: str):
+        with open(path, "wb") as file_stream:
             return dill.dump(data, file_stream)
 
 
 class JsonSerializer(DataSerializer):
     SUFFIX = ".json"
 
-    def load(self, path: Path | str):
-        with open(str(path)) as file_stream:
+    def _load_from(self, path: str):
+        with open(path) as file_stream:
             return json.load(file_stream)
 
-    def dump(self, data: Any, path: Path | str):
-        if hasattr(data, "serialize"):
-            data = data.serialize()
-        with open(str(path), "w") as file_stream:
+    def _dump_to(self, data: Any, path: str):
+        with open(path, "w") as file_stream:
             return json.dump(data, file_stream)
 
 
 class YamlSerializer(DataSerializer):
     SUFFIX = ".yaml"
 
-    def load(self, path: Path | str):
-        with open(str(path)) as file_stream:
+    def _load_from(self, path: str):
+        with open(path) as file_stream:
             return yaml.full_load(file_stream)
 
-    def dump(self, data: Any, path: Path | str):
-        if hasattr(data, "serialize"):
-            data = data.serialize()
-        with open(str(path), "w") as file_stream:
-            return yaml.dump(data, file_stream)
+    def _dump_to(self, data: Any, path: str):
+        with open(path, "w") as file_stream:
+            try:
+                return yaml.safe_dump(data, file_stream)
+            except yaml.representer.RepresenterError as e:
+                raise TypeError(
+                    f"You should return objects that are JSON-serializable. The object {e.args[1]} of type {type(e.args[1])} is not."
+                ) from e
 
 
 SerializerMapping = {
